#ifndef NO_RCSID
char* randlines_rcsid = "$Header: /u/drspeech/src/utils/randlines/RCS/randlines.c,v 1.7 1998/05/25 19:49:54 janin Exp $";
#endif

/*
** Line randomization program
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define MAXLINELEN (32000)

extern void permute(int *vec, int dim);
extern int rseed(int seed);

void get_comline(int,char**);
void usage(void);

char* duplicate_string(char*);

char *fname = "-";
int seed = 0;
char *progname;

int
main(int argc, char* argv[])
{
  FILE *fin;
  int i;
  int num_files;
  int max_files;
  char **files;
  int *order;
  char buf[MAXLINELEN+1];

  progname = argv[0];

  get_comline(argc,argv);

  /* read in the list of files */
  if (strcmp(fname, "-")==0)
  {
      fin = stdin;
  }
  else
  {
      if ((fin = fopen(fname, "r")) == NULL) {
	  fprintf(stderr, "%s: ERROR - Could not open '%s'\n", progname, fname);
	  exit(1);
      }
  }

  num_files = 0;
  max_files = 100;
  files = (char**) malloc(max_files * sizeof(char*));
  if (files==NULL)
  {
      fprintf(stderr, "%s: ERROR - insufficient memory\n", progname);
      exit(1);
  }
  while((fgets(buf, MAXLINELEN, fin)) != NULL) {
      files[num_files] = duplicate_string(buf);
      num_files += 1;
      if (num_files>=max_files)
      {
	  max_files += max_files;
	  files = (char**) realloc(files, max_files*sizeof(char*));
	  if (files==NULL)
	  {
	      fprintf(stderr, "%s: ERROR - insufficient memory\n", progname);
	      exit(1);
	  }
      }
  }

  /* print files in random order */
  order = (int *) malloc(num_files * sizeof(int));
  if (order==NULL)
  {
      fprintf(stderr, "%s: ERROR - insufficient memory\n", progname);
      exit(1);
  }
  permute(order, num_files);

  for (i=0; i < num_files; i++) {
    printf("%s", files[order[i]]);
  }
  exit(0);
}


char*
duplicate_string(char* str) {
  char* dup;
  dup = malloc(strlen(str)+1);
  if (dup == NULL) 
    {
      fprintf(stderr, "%s: ERROR - insufficient memory\n", progname);
      exit(1);
    }
  strcpy(dup, str);
  return dup;
}


void
get_comline(int argc, char** argv)
{
    int c;
    extern int optind;
    extern char *optarg;

    while( (c = getopt( argc, argv, "f:s:")) != -1 )
    {
        switch( c )
        {
	case 'f':/* input file */
	  fname = optarg;
	  break;

	case 's':/* seed for random number generator */
	  seed = atoi(optarg);
	  break;

	default:
	  usage();
	  break;
        }
    }

    if (fname == NULL) {
      fprintf(stderr, "Must set input file to randomize\n");
      usage();
    }

    rseed(seed);
}


void
usage(void)
{
    fprintf(stderr,"Usage: %s [-f <file>] [-s <seed>]\n", progname);
    fprintf(stderr,"    -f name of input file [-]\n");
    fprintf(stderr,"    -s seed for random number generator [time]\n");
    exit( 0 );
}
