#ifndef NO_RCSID
char* vector_rcsid = "$Header: /u/drspeech/src/utils/randlines/RCS/vector.c,v 1.3 1996/09/03 20:56:13 davidj Exp $";
#endif

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
  
/* Written by Gary Tajchman  5/20/92 */

void vector_add (vector1, vector2, sum_vector, dimension)
     float vector1[];
     float vector2[];
     float sum_vector[];
     int dimension;

{
  int i;
  
  /* add elements of vector */
  for (i = 0; i < dimension; i++)
    sum_vector[i] = vector1[i] + vector2[i];
}

void vector_sub (vector1, vector2, sub_vector, dimension)
     float vector1[];
     float vector2[];
     float sub_vector[];
     int dimension;

{
  int i;
  
  /* subtract elements of vector */
  for (i = 0; i < dimension; i++)
    sub_vector[i] = vector1[i] - vector2[i];
}

float vector_length (vector, dimension)
     float vector[];
     int dimension;
     
{
  int i;
  float length;
  
  /* calculate [a,a] */
  length = 0.0;
  for (i = 0; i < dimension; i++)
    length += vector[i] * vector[i];
  
  /* length = sqrt [a,a] */
  length = (float) sqrt (length);
  
  return (length);
}

void vector_scalar_mult (vector, scalar, mult_vector, dimension)
     float vector[];
     float scalar;
     float mult_vector[];
     int dimension;
     
{
  int i;
  
  for (i = 0; i < dimension; i++)
    mult_vector[i] = vector[i] * scalar;
  
}

float inner_product (vector1, vector2, dimension)
     float vector1[];
     float vector2[];
     int dimension;
     
{
  int i;
  float product;
  
  /* calculate [a,b] */
  product = 0.0;
  for (i = 0; i < dimension; i++)
    product += vector1[i] * vector2[i];
  
  return (product);
}


void normalize (vector, norm_vector, dimension)
     float vector[];
     float norm_vector[];
     int dimension;
     
{
  int i;
  float length;
  
  length = vector_length (vector, dimension);
  
  /* if vector length is zero, norm_vector = vector */
  if (length == 0)
    for (i = 0; i < dimension; i++)
      norm_vector[i] = vector[i];
  
  /* else norm_vector = normalized vector */
  else
    for (i = 0; i < dimension; i++)
      norm_vector[i] = vector[i] / length;
  
}

float vector_cos (vector1, vector2, dimension)
     float vector1[];
     float vector2[];
     int dimension;
     
{
  float *norm_vector1;
  float *norm_vector2;
  float cosine;
  
  norm_vector1 = (float *) calloc (dimension, sizeof (float));
  norm_vector2 = (float *) calloc (dimension, sizeof (float));
  
  normalize (vector1, norm_vector1, dimension);
  normalize (vector2, norm_vector2, dimension);
  cosine = inner_product (norm_vector1, norm_vector2, dimension);
  
  free(norm_vector1);
  free(norm_vector2);
  
  return (cosine);
}


void matrix_vector_mult(matrix, vector, output, dim)
     
     float *matrix[];
     float vector[];
     float output[];
     int dim;
     
{
  int i;
  
  for (i=0; i < dim; i++)
    output[i] = inner_product(matrix[i], vector, dim);
  
}

float mean_square_error2(x, y, dim)
     
     float *x,*y;
     int dim;
     
{
  float temp;
  float mse = 0.0;
  int i;
  
  for (i=0; i < dim; i++) {
    temp = x[i] - y[i];
    temp = temp * temp;
    mse += temp;
  }
  
  mse = mse / dim;
  
  return(mse);
}

float mean_square_error1(x, dim)
     
     float *x;
     int dim;
     
{
  float mse = 0.0;
  int i;
  
  for (i=0; i < dim; i++) 
    mse += x[i] * x[i];
  
  mse = mse / dim;
  
  return(mse);
}
